/*
 * File:         arch/blackfin/mach-bf533/stamp.c
 * Based on:     arch/blackfin/mach-bf533/ezkit.c
 * Author:       Aidan Williams <aidan@nicta.com.au>
 *
 * Created:      2005
 * Description:  Board Info File for the BF533-STAMP
 *
 * Rev:          $Id: stamp.c,v 1.29.2.1 2006/10/23 09:54:16 jerry Exp $
 *
 * Modified:
 *               Copyright 2005 National ICT Australia (NICTA)
 *               Copyright 2004-2006 Analog Devices Inc.
 *
 * Bugs:         Enter bugs at http://blackfin.uclinux.org/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#if defined(CONFIG_USB_ISP1362_HCD) || defined(CONFIG_USB_ISP1362_HCD_MODULE)
#include <linux/usb_isp1362.h>
#endif
#include <asm/irq.h>
#include <asm/bfin5xx_spi.h>
#include <linux/dm9000.h>
#include <linux/usb_sl811.h>

/*
 * Name the Board for the /proc/cpuinfo
 */
char *bfin_board_name = "ADDS-BF533-STAMP";

/*
 *  Driver needs to know address, irq and flag pin.
 */

#if defined(CONFIG_DM9000)
/*
 *	0--Address Register 1--Data Register 2--IRQ
 */
static struct resource DM9000_01_resources[] = {
        [0] = {
               .start = 0x20300300,
               .end = 0x20300300 + 1,
               .flags = IORESOURCE_MEM,
               },
        [1] = {
               .start = 0x20300304,
               .end = 0x20300304 + 1,
               .flags = IORESOURCE_MEM,
               },
        [2] = {
               /*
                *  denotes the flag pin and is used directly if
                *  CONFIG_IRQCHIP_DEMUX_GPIO is defined.
                */
               .start = IRQ_PF5,
               .end = IRQ_PF5,
               .flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
               },
};

static struct resource DM9000_02_resources[] = {
        [0] = {
               .start = 0x20320300,
               .end = 0x20320300 + 1,
               .flags = IORESOURCE_MEM,
               },
        [1] = {
               .start = 0x20320304,
               .end = 0x20320304 + 1,
               .flags = IORESOURCE_MEM,
               },
        [2] = {
               .start = IRQ_PF6,
               .end = IRQ_PF6,
               .flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
               },
};

static struct dm9000_plat_data bfin_dm9000_platdata = {
.flags = DM9000_PLATF_16BITONLY,
};

static struct platform_device DM9000_01_device = {
        .name = "dm9000",
        .id = 0,
        .num_resources = ARRAY_SIZE(DM9000_01_resources),
        .resource = DM9000_01_resources,
        .dev = {
        .platform_data = &bfin_dm9000_platdata,
        }
};

static struct platform_device DM9000_02_device = {
        .name = "dm9000",
        .id = 1,
        .num_resources = ARRAY_SIZE(DM9000_02_resources),
        .resource = DM9000_02_resources,
        .dev = {
        .platform_data = &bfin_dm9000_platdata,
        }
};

#endif

#if defined(CONFIG_SMC91X) || defined(CONFIG_SMC91X_MODULE)
static struct resource smc91x_resources[] = {
	{
		.name = "smc91x-regs",
		.start = 0x20300300,
		.end = 0x20300300 + 16,
		.flags = IORESOURCE_MEM,
	},{
		.start = IRQ_PROG_INTB,
		.end = IRQ_PROG_INTB,
		.flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
	},{
		/*
		 *  denotes the flag pin and is used directly if
		 *  CONFIG_IRQCHIP_DEMUX_GPIO is defined.
		 */
		.start = IRQ_PF7,
		.end = IRQ_PF7,
		.flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
	},
};

static struct platform_device smc91x_device = {
	.name = "smc91x",
	.id = 0,
	.num_resources = ARRAY_SIZE(smc91x_resources),
	.resource = smc91x_resources,
};
#endif

#if defined(CONFIG_USB_NET2272) || defined(CONFIG_USB_NET2272_MODULE)
static struct resource net2272_bfin_resources[] = {
	{
		.start = 0x20300000,
		.end = 0x20300000 + 0x100,
		.flags = IORESOURCE_MEM,
	},{
		.start = IRQ_PF10,
		.end = IRQ_PF10,
		.flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
	},
};

static struct platform_device net2272_bfin_device = {
	.name = "net2272",
	.id = -1,
	.num_resources = ARRAY_SIZE(net2272_bfin_resources),
	.resource = net2272_bfin_resources,
};
#endif

/*
 * SL811 USB
 */

#if defined(CONFIG_USB_SL811_HCD) || defined(CONFIG_USB_SL811_HCD_MODULE)
static struct resource sl811_hcd_resources[] = {
        [0] = {
               .start = 0x20000000,
               .end = 0x20000000,
               .flags = IORESOURCE_MEM,
               },
        [1] = {
               .start = 0x20000004,
               .end = 0x20000004,
               .flags = IORESOURCE_MEM,
               },
        [2] = {
                .start = IRQ_PROG_INTA,
                .end = IRQ_PROG_INTA,
                .flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
                },
/*
        [3] = {
               .start = IRQ_PF0 + CONFIG_USB_SL811_BFIN_GPIO,
               .end = IRQ_PF0 + CONFIG_USB_SL811_BFIN_GPIO,
               .flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
               },
*/

       [3] = {
               .start = IRQ_PF7, //+ CONFIG_USB_SL811_BFIN_GPIO,
               .end = IRQ_PF7, // + CONFIG_USB_SL811_BFIN_GPIO,
               .flags = IORESOURCE_IRQ | IORESOURCE_IRQ_HIGHLEVEL,
               },

};

#if defined(CONFIG_USB_SL811_BFIN_USE_VBUS)
void sl811_port_power(struct device *dev, int is_on)
{
	unsigned short pFD;
	unsigned short mask = (1<<CONFIG_USB_SL811_BFIN_GPIO_VBUS);
	
	pFD = bfin_read_FIO_DIR();
	bfin_write_FIO_DIR( (pFD | mask) );
//	*pFIO_DIR |= mask;

	if(is_on)
		bfin_write_FIO_FLAG_S(mask);
//		*pFIO_FLAG_S = mask;
        else
		bfin_write_FIO_FLAG_C(mask);
//		*pFIO_FLAG_C = mask;
}
#endif

static struct sl811_platform_data sl811_priv = {
        .potpg          = 10,
        .power          = 250,          /* == 500mA */
#if defined(CONFIG_USB_SL811_BFIN_USE_VBUS)
        .port_power = &sl811_port_power,
#endif
};

static struct platform_device sl811_hcd_device = {
        .name = "sl811-hcd",
        .id = 0,
        .dev = {
                .platform_data = &sl811_priv,
        },
        .num_resources = ARRAY_SIZE(sl811_hcd_resources),
        .resource = sl811_hcd_resources,
};

#endif


#if defined(CONFIG_SPI_BFIN) || defined(CONFIG_SPI_BFIN_MODULE)
/* all SPI peripherals info goes here */

#if defined(CONFIG_MTD_M25P80) || defined(CONFIG_MTD_M25P80_MODULE)
static struct mtd_partition bfin_spi_flash_partitions[] = {
	{
		.name = "bootloader",
		.size = 0x00040000,
		.offset = 0,
		.mask_flags = MTD_CAP_ROM
	},{
		.name = "kernel",
		.size = 0xc0000,
		.offset = 0x40000
	},{
		.name = "file system",
		.size = 0x300000,
		.offset = 0x00100000,
	}
};

static struct flash_platform_data bfin_spi_flash_data = {
	.name = "m25p80",
	.parts = bfin_spi_flash_partitions,
	.nr_parts = ARRAY_SIZE(bfin_spi_flash_partitions),
	.type = "m25p64",
};

/* SPI flash chip (m25p64) */
static struct bfin5xx_spi_chip spi_flash_chip_info = {
	.ctl_reg = 0x1C00,       /* with enable bit unset */
	.enable_dma = 0,         /* use dma transfer with this chip*/
	.bits_per_word = 8,
};
#endif

#if defined(CONFIG_SPI_ADC_BF533) || defined(CONFIG_SPI_ADC_BF533_MODULE)
/* SPI ADC chip */
static struct bfin5xx_spi_chip spi_adc_chip_info = {
	.ctl_reg = 0x1000,
	.enable_dma = 1,         /* use dma transfer with this chip*/
	.bits_per_word = 16,
};
#endif

#if defined(CONFIG_SND_BLACKFIN_AD1836) || defined(CONFIG_SND_BLACKFIN_AD1836_MODULE)
static struct bfin5xx_spi_chip ad1836_spi_chip_info = {
	.ctl_reg = 0x1000,
	.enable_dma = 0,
	.bits_per_word = 16,
};
#endif

#if defined(CONFIG_PBX)
static struct bfin5xx_spi_chip spi_si3xxx_chip_info = {
	.ctl_reg	= 0x1c04,
	.enable_dma	= 0,
	.bits_per_word	= 8,
	.cs_change_per_word = 1,
};
#endif

/* Notice: for blackfin, the speed_hz is the value of register
 * SPI_BAUD, not the real baudrate */
static struct spi_board_info bfin_spi_board_info[] __initdata = {
#if defined(CONFIG_MTD_M25P80) || defined(CONFIG_MTD_M25P80_MODULE)
	{
		/* the modalias must be the same as spi device driver name */
		.modalias = "m25p80", /* Name of spi_driver for this device */
		/* this value is the baudrate divisor */
		.max_speed_hz = 2,     /* actual baudrate is SCLK/(2xspeed_hz) */
		.bus_num = 1, /* Framework bus number */
		.chip_select = 2, /* Framework chip select. On STAMP537 it is SPISSEL2*/
		.platform_data = &bfin_spi_flash_data,
		.controller_data = &spi_flash_chip_info,
	},
#endif

#if defined(CONFIG_SPI_ADC_BF533) || defined(CONFIG_SPI_ADC_BF533_MODULE)
	{
		.modalias = "bfin_spi_adc", /* Name of spi_driver for this device */
		.max_speed_hz = 4,     /* actual baudrate is SCLK/(2xspeed_hz) */
		.bus_num = 1, /* Framework bus number */
		.chip_select = 1, /* Framework chip select. */
		.platform_data = NULL, /* No spi_driver specific config */
		.controller_data = &spi_adc_chip_info,
	},
#endif

#if defined(CONFIG_SND_BLACKFIN_AD1836) || defined(CONFIG_SND_BLACKFIN_AD1836_MODULE)
	{
		.modalias = "ad1836-spi",
		.max_speed_hz = 16,
		.bus_num = 1,
		.chip_select = CONFIG_SND_BLACKFIN_SPI_PFBIT,
		.controller_data = &ad1836_spi_chip_info,
	},
#endif

#if defined(CONFIG_PBX)
	{
		.modalias	= "fxs-spi",
		.max_speed_hz	= 4,
		.bus_num	= 1,
		.chip_select	= 3,
		.controller_data= &spi_si3xxx_chip_info,
	},

	{
		.modalias	= "fxo-spi",
		.max_speed_hz	= 4,
		.bus_num	= 1,
		.chip_select	= 2,
		.controller_data= &spi_si3xxx_chip_info,
	},
#endif
};

/* SPI controller data */
static struct bfin5xx_spi_master spi_bfin_master_info = {
	.num_chipselect = 8,
	.enable_dma = 1,  /* master has the ability to do dma transfer */
};

static struct platform_device spi_bfin_master_device = {
	.name = "bfin-spi-master",
	.id = 1, /* Bus number */
	.dev = {
		.platform_data = &spi_bfin_master_info, /* Passed to driver */
	},
};
#endif  /* spi master and devices */

#if defined(CONFIG_FB_BF537_LQ035) || defined(CONFIG_FB_BF537_LQ035_MODULE)
static struct platform_device bfin_fb_device = {
	.name = "bf537-fb",
};
#endif

#if defined(CONFIG_SERIAL_BFIN) || defined(CONFIG_SERIAL_BFIN_MODULE)
static struct resource bfin_uart_resources[] = {
	{
		.start = 0xFFC00400,
		.end = 0xFFC004FF,
		.flags = IORESOURCE_MEM,
	},
};

static struct platform_device bfin_uart_device = {
	.name = "bfin-uart",
	.id = 1,
	.num_resources = ARRAY_SIZE(bfin_uart_resources),
	.resource = bfin_uart_resources,
};
#endif

static struct platform_device *stamp_devices[] __initdata = {
#if defined(CONFIG_DM9000)
	&DM9000_01_device,
	&DM9000_02_device,
#endif

#if defined(CONFIG_USB_SL811_HCD) || defined(CONFIG_USB_SL811_HCD_MODULE)
        &sl811_hcd_device,
#endif

#if defined(CONFIG_SMC91X) || defined(CONFIG_SMC91X_MODULE)
	&smc91x_device,
#endif

#if defined(CONFIG_USB_NET2272) || defined(CONFIG_USB_NET2272_MODULE)
	&net2272_bfin_device,
#endif

#if defined(CONFIG_SPI_BFIN) || defined(CONFIG_SPI_BFIN_MODULE)
	&spi_bfin_master_device,
#endif

#if defined(CONFIG_SERIAL_BFIN) || defined(CONFIG_SERIAL_BFIN_MODULE)
	&bfin_uart_device,
#endif
};

static int __init stamp_init(void)
{
	printk(KERN_INFO "%s(): registering device resources\n", __FUNCTION__);
	platform_add_devices(stamp_devices, ARRAY_SIZE(stamp_devices));
#if defined(CONFIG_SPI_BFIN) || defined(CONFIG_SPI_BFIN_MODULE)
	spi_register_board_info(bfin_spi_board_info, ARRAY_SIZE(bfin_spi_board_info));
#endif
	return 0;
}

arch_initcall(stamp_init);
